﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectExcavator.DrawingObjects;
using ProjectExcavator.Exceptions;
using ProjectExcavator.Generics;
using ProjectExcavator.MovementStrategy;

namespace ProjectExcavator.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class ExcavatorGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, ExcavatorGenericCollection<DrawingExcavator,
        DrawingObjectExcavator>> _excavatorStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _excavatorStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        public ExcavatorGenericStorage(int pictureWidth, int pictureHeight)
        {
            _excavatorStorages = new Dictionary<string,
            ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_excavatorStorages.ContainsKey(name))
            {
                ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator> newSet = new ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>(_pictureWidth, _pictureHeight);
                _excavatorStorages.Add(name, newSet);
            }
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            {
                if (_excavatorStorages.ContainsKey(name))
                {
                    _excavatorStorages.Remove(name);
                }
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>? this[string ind]
        {
            get
            {
                if (_excavatorStorages.ContainsKey(ind))
                {
                    return _excavatorStorages[ind];
                }
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации по экскаваторам в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>> record in _excavatorStorages)
            {
                StringBuilder records = new();
                foreach (DrawingExcavator? elem in record.Value.GetExcavators)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new ArgumentException("Нет данных для сохранения");
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.WriteLine("excavatorStorages");
                writer.Write(data.ToString());
            }
        }
        /// <summary>
        /// Загрузка информации по экскаваторам в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }

            using (StreamReader reader = new StreamReader(filename))
            {
                string proverkaline = reader.ReadLine();
                if (proverkaline == null)
                {
                    throw new ArgumentException("Нет данных для загрузки");
                }
                if (!proverkaline.StartsWith("excavatorStorages"))
                {
                    throw new InvalidDataException("Неверный формат ввода файла");
                }

                _excavatorStorages.Clear();

                string line;
                while ((line = reader.ReadLine()) != null)
                {
                    string[] parts = line.Split('|');
                    if (parts.Length != 2)
                    {
                        continue;
                    }

                    string namestorage = parts[0];
                    ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator> collection = new(_pictureWidth, _pictureHeight);

                    foreach (string data in parts[1].Split(';'))
                    {
                        DrawingExcavator? excavator = data?.CreateDrawingExcavator(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (excavator != null)
                        {
                            try { _ = collection + excavator; }
                            catch (ExcavatorNotFoundException ex)
                            {
                                throw ex;
                            }
                            catch (StorageOverflowException ex)
                            {
                                throw ex;
                            }
                        }
                    }

                    _excavatorStorages.Add(namestorage, collection);
                }
            }
        }
    }
}
