﻿using ProjectExcavator.DrawingObjects;
using ProjectExcavator.Generics;
using ProjectExcavator.Exceptions;
using Microsoft.Extensions.Logging;
using ProjectExcavator.MovementStrategy;
using ProjectExcavator.Excavators;
using System.Linq.Expressions;
using System.Xml.Linq;


namespace ProjectExcavator
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawningExcavator
    /// </summary>
    public partial class FormExcavatorCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly ExcavatorGenericStorage _storage;
        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormExcavatorCollection(ILogger<FormExcavatorCollection> logger)
        {
            InitializeComponent();
            _storage = new ExcavatorGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;
        }
        /// <summary>
        /// Заполнение listBoxObjects
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
            >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
                var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
                if (obj == null)
                {
                    return;
                }
                pictureBoxCollection.Image = obj.ShowExcavator();
            }
        }
        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Пустое название набора");
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Добавлен набор: {textBoxStorageName.Text}");
        }
        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxObjects_SelectedIndexChanged(object sender,
        EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowExcavator();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление невыбранного набора");
                return;
            }
            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Удалить объект {name}?", "Удаление", MessageBoxButtons.YesNo,
        MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(name);
                ReloadObjects();
                _logger.LogInformation($"Удален набор: {name}");
            }
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddEx_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                _logger.LogWarning("Добавление пустого объекта");
                return;
            }
            FormExcavatorConfig form = new FormExcavatorConfig();
            Action<DrawingExcavator> ExcavatorDelegate = new Action<DrawingExcavator>((excavator) =>
            {
                try
                {
                    bool selectedexcavator = obj + excavator;
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowExcavator();
                    _logger.LogInformation($"Добавлен объект в набор {listBoxStorages.SelectedItem.ToString()}");
                }
                catch (StorageOverflowException ex)
                {
                    MessageBox.Show("Не удалось добавить объект");
                    _logger.LogWarning($"Не удалось добавить объект: {ex.Message}");
                }
            });
            form.AddEvent(ExcavatorDelegate);
            form.Show();
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveEx_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление объекта из несуществующего набора");
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            try
            {
                if (obj - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowExcavator();
                    _logger.LogInformation($"Удален объект из набора {listBoxStorages.SelectedItem.ToString()}");
                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    _logger.LogWarning($"Не удалось удалить объект из набора {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (ExcavatorNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"{ex.Message} из набора {listBoxStorages.SelectedItem.ToString()}");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowExcavator();
        }
        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Сохранение наборов в файл {saveFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось сохранить наборы с ошибкой: {ex.Message}");
                }
            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // TODO продумать логику
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Загрузились наборы из файла {openFileDialog.FileName}");
                    ReloadObjects();
                    if (listBoxStorages.SelectedIndex == -1)
                    {
                        return;
                    }
                    var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
                    string.Empty];
                    if (obj == null)
                    {
                        return;
                    }
                    pictureBoxCollection.Image = obj.ShowExcavator();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Не загрузилось", "Результат",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось сохранить наборы с ошибкой: {ex.Message}");
                }
            }
        }
    }
}
